@@ -976,16 +976,18 @@ See `completion-in-region' for the arguments BEG, END, TABLE, PRED."
               (cands (alist-get 'corfu--candidates state)))
          (cond
           ((<= total 1)
-           ;; If completion is finished and cannot be extended further and
-           ;; `corfu-on-exact-match' is not 'show, return 'finished.  Otherwise
-           ;; setup the popup.
-           (if (and (= total 1)
-                    (or (eq corfu-on-exact-match 'show)
-                        (consp (completion-try-completion
-                                newstr table pred newpt
-                                (completion-metadata newstr table pred)))))
-               (corfu--setup beg end table pred)
-             (corfu--exit-function newstr 'finished cands)))
+           (cond
+            ;; Setup popup if completion can continue or if
+            ;; `corfu-on-exact-match' is `show'.
+            ((and (= total 1)
+                  (or (eq corfu-on-exact-match 'show)
+                      (consp (completion-try-completion
+                              newstr table pred newpt
+                              (completion-metadata newstr table pred)))))
+             (corfu--setup beg end table pred))
+            ;; Otherwise exit completion with status `finished'.
+            ((test-completion newstr table pred)
+             (corfu--exit-function newstr 'finished cands))))
           ;; Too many candidates for cycling -> Setup popup.
           ((or (not threshold) (and (not (eq threshold t)) (< threshold total)))
            (corfu--setup beg end table pred))
@@ -1354,16 +1356,16 @@ If a candidate is selected, insert it.  Otherwise invoke
   (if (< corfu--index 0)
       (corfu-expand)
     ;; Continue completion with selected candidate.  Exit with status 'finished
-    ;; if input is a valid match and no further completion is
-    ;; possible. Additionally treat completion as finished if at the end of a
-    ;; boundary, even if other longer candidates would still match, since the
-    ;; user invoked `corfu-complete' with an explicitly selected candidate!
+    ;; if input is a valid match and no further completion is possible.
     (pcase-let ((`(,_beg ,_end ,table ,pred . ,_) completion-in-region--data)
                 (newstr (corfu--insert nil)))
       (and (test-completion newstr table pred)
            (or (not (consp (completion-try-completion
                             newstr table pred (length newstr)
                             (completion-metadata newstr table pred))))
+               ;; Additionally finish completion if at the end of a boundary,
+               ;; even if other longer candidates match, since the user invoked
+               ;; `corfu-complete' with an explicitly selected candidate!
                (equal (completion-boundaries newstr table pred "") '(0 . 0)))
            (corfu--done newstr 'finished nil))
       t)))
